
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "gvc-enum-types.h"
#include "gvc-mixer-stream.h"

GType
gvc_mixer_stream_state_get_type (void)
{
  static gsize g_enum_type_id;
  static const GEnumValue values[] =
    {
      { GVC_STREAM_STATE_INVALID, "GVC_STREAM_STATE_INVALID", "invalid" },
      { GVC_STREAM_STATE_RUNNING, "GVC_STREAM_STATE_RUNNING", "running" },
      { GVC_STREAM_STATE_IDLE, "GVC_STREAM_STATE_IDLE", "idle" },
      { GVC_STREAM_STATE_SUSPENDED, "GVC_STREAM_STATE_SUSPENDED", "suspended" },
      { 0, NULL, NULL }
    };

  if (g_once_init_enter (&g_enum_type_id))
    {
      const gchar *string;
      GType id;

      string = g_intern_static_string ("GvcMixerStreamState");
      id = g_enum_register_static (string, values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* Generated data ends here */

