
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "gf-desktop-enum-types.h"
#include "gf-desktop-enums.h"

GType
gf_icon_size_get_type (void)
{
  static gsize g_enum_type_id;
  static const GEnumValue values[] =
    {
      { GF_ICON_SIZE_16PX, "GF_ICON_SIZE_16PX", "16px" },
      { GF_ICON_SIZE_22PX, "GF_ICON_SIZE_22PX", "22px" },
      { GF_ICON_SIZE_24PX, "GF_ICON_SIZE_24PX", "24px" },
      { GF_ICON_SIZE_32PX, "GF_ICON_SIZE_32PX", "32px" },
      { GF_ICON_SIZE_48PX, "GF_ICON_SIZE_48PX", "48px" },
      { GF_ICON_SIZE_64PX, "GF_ICON_SIZE_64PX", "64px" },
      { GF_ICON_SIZE_72PX, "GF_ICON_SIZE_72PX", "72px" },
      { GF_ICON_SIZE_96PX, "GF_ICON_SIZE_96PX", "96px" },
      { GF_ICON_SIZE_128PX, "GF_ICON_SIZE_128PX", "128px" },
      { 0, NULL, NULL }
    };

  if (g_once_init_enter (&g_enum_type_id))
    {
      const gchar *string;
      GType id;

      string = g_intern_static_string ("GfIconSize");
      id = g_enum_register_static (string, values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

GType
gf_placement_get_type (void)
{
  static gsize g_enum_type_id;
  static const GEnumValue values[] =
    {
      { GF_PLACEMENT_AUTO_ARRANGE_ICONS, "GF_PLACEMENT_AUTO_ARRANGE_ICONS", "auto-arrange-icons" },
      { GF_PLACEMENT_ALIGN_ICONS_TO_GRID, "GF_PLACEMENT_ALIGN_ICONS_TO_GRID", "align-icons-to-grid" },
      { GF_PLACEMENT_FREE, "GF_PLACEMENT_FREE", "free" },
      { 0, NULL, NULL }
    };

  if (g_once_init_enter (&g_enum_type_id))
    {
      const gchar *string;
      GType id;

      string = g_intern_static_string ("GfPlacement");
      id = g_enum_register_static (string, values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

GType
gf_sort_by_get_type (void)
{
  static gsize g_enum_type_id;
  static const GEnumValue values[] =
    {
      { GF_SORT_BY_NAME, "GF_SORT_BY_NAME", "name" },
      { GF_SORT_BY_DATE_MODIFIED, "GF_SORT_BY_DATE_MODIFIED", "date-modified" },
      { GF_SORT_BY_SIZE, "GF_SORT_BY_SIZE", "size" },
      { 0, NULL, NULL }
    };

  if (g_once_init_enter (&g_enum_type_id))
    {
      const gchar *string;
      GType id;

      string = g_intern_static_string ("GfSortBy");
      id = g_enum_register_static (string, values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* Generated data ends here */

