
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "gf-screensaver-enum-types.h"
#include "gf-auth.h"

GType
gf_auth_message_type_get_type (void)
{
  static gsize g_enum_type_id;
  static const GEnumValue values[] =
    {
      { GF_AUTH_MESSAGE_PROMPT_ECHO_OFF, "GF_AUTH_MESSAGE_PROMPT_ECHO_OFF", "prompt-echo-off" },
      { GF_AUTH_MESSAGE_PROMPT_ECHO_ON, "GF_AUTH_MESSAGE_PROMPT_ECHO_ON", "prompt-echo-on" },
      { GF_AUTH_MESSAGE_ERROR_MSG, "GF_AUTH_MESSAGE_ERROR_MSG", "error-msg" },
      { GF_AUTH_MESSAGE_TEXT_INFO, "GF_AUTH_MESSAGE_TEXT_INFO", "text-info" },
      { 0, NULL, NULL }
    };

  if (g_once_init_enter (&g_enum_type_id))
    {
      const gchar *string;
      GType id;

      string = g_intern_static_string ("GfAuthMessageType");
      id = g_enum_register_static (string, values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
#include "gf-unlock-dialog.h"

GType
gf_unlock_dialog_response_get_type (void)
{
  static gsize g_enum_type_id;
  static const GEnumValue values[] =
    {
      { GF_UNLOCK_DIALOG_RESPONSE_NONE, "GF_UNLOCK_DIALOG_RESPONSE_NONE", "none" },
      { GF_UNLOCK_DIALOG_RESPONSE_OK, "GF_UNLOCK_DIALOG_RESPONSE_OK", "ok" },
      { GF_UNLOCK_DIALOG_RESPONSE_CANCEL, "GF_UNLOCK_DIALOG_RESPONSE_CANCEL", "cancel" },
      { 0, NULL, NULL }
    };

  if (g_once_init_enter (&g_enum_type_id))
    {
      const gchar *string;
      GType id;

      string = g_intern_static_string ("GfUnlockDialogResponse");
      id = g_enum_register_static (string, values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* Generated data ends here */

