gf-screensaver-panel {
  font-weight: bold;
  padding: 4px 10px;
  background: rgba (0, 0, 0, 0.5);
  color: #cccccc;
}

gf-screensaver-panel image,
gf-screensaver-panel label {
  color: #cccccc;
}

gf-unlock-dialog,
gf-unlock-dialog frame border {
  border-radius: 10px;
}

gf-popup-window {
  border-radius: 10px;
}

gf-popup-window.solid {
  border-radius: 0px;
}

#gf-candidate-popup {
  border-radius: 3px;
}

#gf-candidate-popup.solid {
  border-radius: 0px;
}

#gf-candidate-popup gf-candidate-box label {
  padding: 4px 6px;
}

#gf-candidate-popup gf-candidate-box label:last-child {
  padding-left: 0px;
}

#gf-candidate-popup gf-candidate-box:selected,
#gf-candidate-popup gf-candidate-box:hover {
  border-radius: 3px;
}

#gf-input-source-popup #gf-input-source {
  min-width: 120px;
  min-height: 120px;
  font-size: 22px;
}

#gf-input-source-popup #gf-input-source:selected {
  border-radius: 6px;
}

#gf-label-window {
  font-size: 40px;
}

gf-icon {
  margin: 2px;
}

gf-icon label {
  border-radius: 3px;
  text-shadow: 1px 1px black;
  padding: 2px;
}

gf-icon:hover image {
  -gtk-icon-effect: highlight;
}
