/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GF_DBUS_X11_H__
#define __GF_DBUS_X11_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Mutter.X11 */

#define GF_DBUS_TYPE_X11 (gf_dbus_x11_get_type ())
#define GF_DBUS_X11(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GF_DBUS_TYPE_X11, GfDBusX11))
#define GF_DBUS_IS_X11(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GF_DBUS_TYPE_X11))
#define GF_DBUS_X11_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GF_DBUS_TYPE_X11, GfDBusX11Iface))

struct _GfDBusX11;
typedef struct _GfDBusX11 GfDBusX11;
typedef struct _GfDBusX11Iface GfDBusX11Iface;

struct _GfDBusX11Iface
{
  GTypeInterface parent_iface;

  gint  (*get_ui_scaling_factor) (GfDBusX11 *object);

};

GType gf_dbus_x11_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gf_dbus_x11_interface_info (void);
guint gf_dbus_x11_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
gint gf_dbus_x11_get_ui_scaling_factor (GfDBusX11 *object);
void gf_dbus_x11_set_ui_scaling_factor (GfDBusX11 *object, gint value);


/* ---- */

#define GF_DBUS_TYPE_X11_PROXY (gf_dbus_x11_proxy_get_type ())
#define GF_DBUS_X11_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GF_DBUS_TYPE_X11_PROXY, GfDBusX11Proxy))
#define GF_DBUS_X11_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GF_DBUS_TYPE_X11_PROXY, GfDBusX11ProxyClass))
#define GF_DBUS_X11_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GF_DBUS_TYPE_X11_PROXY, GfDBusX11ProxyClass))
#define GF_DBUS_IS_X11_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GF_DBUS_TYPE_X11_PROXY))
#define GF_DBUS_IS_X11_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GF_DBUS_TYPE_X11_PROXY))

typedef struct _GfDBusX11Proxy GfDBusX11Proxy;
typedef struct _GfDBusX11ProxyClass GfDBusX11ProxyClass;
typedef struct _GfDBusX11ProxyPrivate GfDBusX11ProxyPrivate;

struct _GfDBusX11Proxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GfDBusX11ProxyPrivate *priv;
};

struct _GfDBusX11ProxyClass
{
  GDBusProxyClass parent_class;
};

GType gf_dbus_x11_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GfDBusX11Proxy, g_object_unref)
#endif

void gf_dbus_x11_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GfDBusX11 *gf_dbus_x11_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GfDBusX11 *gf_dbus_x11_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gf_dbus_x11_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GfDBusX11 *gf_dbus_x11_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GfDBusX11 *gf_dbus_x11_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GF_DBUS_TYPE_X11_SKELETON (gf_dbus_x11_skeleton_get_type ())
#define GF_DBUS_X11_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GF_DBUS_TYPE_X11_SKELETON, GfDBusX11Skeleton))
#define GF_DBUS_X11_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GF_DBUS_TYPE_X11_SKELETON, GfDBusX11SkeletonClass))
#define GF_DBUS_X11_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GF_DBUS_TYPE_X11_SKELETON, GfDBusX11SkeletonClass))
#define GF_DBUS_IS_X11_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GF_DBUS_TYPE_X11_SKELETON))
#define GF_DBUS_IS_X11_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GF_DBUS_TYPE_X11_SKELETON))

typedef struct _GfDBusX11Skeleton GfDBusX11Skeleton;
typedef struct _GfDBusX11SkeletonClass GfDBusX11SkeletonClass;
typedef struct _GfDBusX11SkeletonPrivate GfDBusX11SkeletonPrivate;

struct _GfDBusX11Skeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GfDBusX11SkeletonPrivate *priv;
};

struct _GfDBusX11SkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gf_dbus_x11_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GfDBusX11Skeleton, g_object_unref)
#endif

GfDBusX11 *gf_dbus_x11_skeleton_new (void);


G_END_DECLS

#endif /* __GF_DBUS_X11_H__ */
