/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GF_DM_SEAT_GEN_H__
#define __GF_DM_SEAT_GEN_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.DisplayManager.Seat */

#define GF_TYPE_DM_SEAT_GEN (gf_dm_seat_gen_get_type ())
#define GF_DM_SEAT_GEN(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GF_TYPE_DM_SEAT_GEN, GfDmSeatGen))
#define GF_IS_DM_SEAT_GEN(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GF_TYPE_DM_SEAT_GEN))
#define GF_DM_SEAT_GEN_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GF_TYPE_DM_SEAT_GEN, GfDmSeatGenIface))

struct _GfDmSeatGen;
typedef struct _GfDmSeatGen GfDmSeatGen;
typedef struct _GfDmSeatGenIface GfDmSeatGenIface;

struct _GfDmSeatGenIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_switch_to_greeter) (
    GfDmSeatGen *object,
    GDBusMethodInvocation *invocation);

  gboolean  (*get_can_switch) (GfDmSeatGen *object);

};

GType gf_dm_seat_gen_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gf_dm_seat_gen_interface_info (void);
guint gf_dm_seat_gen_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void gf_dm_seat_gen_complete_switch_to_greeter (
    GfDmSeatGen *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void gf_dm_seat_gen_call_switch_to_greeter (
    GfDmSeatGen *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gf_dm_seat_gen_call_switch_to_greeter_finish (
    GfDmSeatGen *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gf_dm_seat_gen_call_switch_to_greeter_sync (
    GfDmSeatGen *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean gf_dm_seat_gen_get_can_switch (GfDmSeatGen *object);
void gf_dm_seat_gen_set_can_switch (GfDmSeatGen *object, gboolean value);


/* ---- */

#define GF_TYPE_DM_SEAT_GEN_PROXY (gf_dm_seat_gen_proxy_get_type ())
#define GF_DM_SEAT_GEN_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GF_TYPE_DM_SEAT_GEN_PROXY, GfDmSeatGenProxy))
#define GF_DM_SEAT_GEN_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GF_TYPE_DM_SEAT_GEN_PROXY, GfDmSeatGenProxyClass))
#define GF_DM_SEAT_GEN_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GF_TYPE_DM_SEAT_GEN_PROXY, GfDmSeatGenProxyClass))
#define GF_IS_DM_SEAT_GEN_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GF_TYPE_DM_SEAT_GEN_PROXY))
#define GF_IS_DM_SEAT_GEN_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GF_TYPE_DM_SEAT_GEN_PROXY))

typedef struct _GfDmSeatGenProxy GfDmSeatGenProxy;
typedef struct _GfDmSeatGenProxyClass GfDmSeatGenProxyClass;
typedef struct _GfDmSeatGenProxyPrivate GfDmSeatGenProxyPrivate;

struct _GfDmSeatGenProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GfDmSeatGenProxyPrivate *priv;
};

struct _GfDmSeatGenProxyClass
{
  GDBusProxyClass parent_class;
};

GType gf_dm_seat_gen_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GfDmSeatGenProxy, g_object_unref)
#endif

void gf_dm_seat_gen_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GfDmSeatGen *gf_dm_seat_gen_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GfDmSeatGen *gf_dm_seat_gen_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gf_dm_seat_gen_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GfDmSeatGen *gf_dm_seat_gen_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GfDmSeatGen *gf_dm_seat_gen_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GF_TYPE_DM_SEAT_GEN_SKELETON (gf_dm_seat_gen_skeleton_get_type ())
#define GF_DM_SEAT_GEN_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GF_TYPE_DM_SEAT_GEN_SKELETON, GfDmSeatGenSkeleton))
#define GF_DM_SEAT_GEN_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GF_TYPE_DM_SEAT_GEN_SKELETON, GfDmSeatGenSkeletonClass))
#define GF_DM_SEAT_GEN_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GF_TYPE_DM_SEAT_GEN_SKELETON, GfDmSeatGenSkeletonClass))
#define GF_IS_DM_SEAT_GEN_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GF_TYPE_DM_SEAT_GEN_SKELETON))
#define GF_IS_DM_SEAT_GEN_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GF_TYPE_DM_SEAT_GEN_SKELETON))

typedef struct _GfDmSeatGenSkeleton GfDmSeatGenSkeleton;
typedef struct _GfDmSeatGenSkeletonClass GfDmSeatGenSkeletonClass;
typedef struct _GfDmSeatGenSkeletonPrivate GfDmSeatGenSkeletonPrivate;

struct _GfDmSeatGenSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GfDmSeatGenSkeletonPrivate *priv;
};

struct _GfDmSeatGenSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gf_dm_seat_gen_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GfDmSeatGenSkeleton, g_object_unref)
#endif

GfDmSeatGen *gf_dm_seat_gen_skeleton_new (void);


G_END_DECLS

#endif /* __GF_DM_SEAT_GEN_H__ */
