@import url("resource:///org/gnome/gnome-flashback/theme/common.css");

gf-popup-window {
  background-color: #000000;
  color: #ffffff;
  border: 1px solid rgb(115, 115, 115);
}

gf-popup-window.solid {
  border: 1px solid #8d8d8d;
}

#gf-candidate-popup gf-candidate-box:selected,
#gf-candidate-popup gf-candidate-box:hover {
  background-color: #ffffff;
  color: #000000;
}

#gf-input-source-popup #gf-input-source:selected {
  background-color: #ffffff;
  color: #000000;
}

#gf-osd-window levelbar block.overdrive {
  background-color: #ef2929;
  border-color: #ef2929;
}

gf-icon image,
gf-icon label {
  color: #ffffff;
}

gf-icon:selected label {
  background: #000000;
  color: #ffffff;
}
